import collections
import redis
import time

from django.conf import settings
from django.contrib.contenttypes.models import ContentType


def build_value(instance):
    return '%d:%d' % (ContentType.objects.get_for_model(instance).id, instance.id)


def factory_object(ct_obj_string):
    """
    Utility function that takes a content_type:object_id
    string and returns a django object.

      @ct_obj_string => '1:22'  ('content_type_id:object_id')

    """
    # Source: http://git.io/2tCSwQ
    ct_id, obj_id = ct_obj_string.split(':')
    return ContentType.objects.get_for_id(ct_id).model_class().objects.get(pk=obj_id)


def get_redis_connection():
    return redis.Redis(**settings.REDIS['default'])


def construct_list_using_index(start=0, num=0, storage_key=None):
    redis = get_redis_connection()
    num = num - 1  # `num` is 0-indexed, we specify a 1-index to eliminate confusion.
    redis_results = redis.zrevrange(storage_key, start=start, num=num)

    mapping = collections.defaultdict(list)
    for x in redis_results:
        ct_id, obj_id = x.split(':')
        mapping[ct_id].append(obj_id)

    final = []
    for ct_id, obj_ids in mapping.iteritems():
        final.extend(ContentType.objects.get_for_id(ct_id).model_class().objects.filter(pk__in=obj_ids))
    return final


def construct_list_using_score(min=0, max=0, storage_key=None):
    redis = get_redis_connection()
    redis_results = redis.zrevrangebyscore(storage_key, min=0, max=0)
    return [factory_object(ct_obj_string) for ct_obj_string in redis_results]


def paginate_list(page=1, results_per_page=20, storage_key=None):
    redis = get_redis_connection()
    max_score = int(time.time())
    count = redis.zcount(storage_key, min=0, max=max_score)

    # Calculate the 'start' index.
    start = count - (results_per_page * page)
    if start < 0:
        start = 0

    # Fetch a slice of the results for the page given.
    redis_results = redis.zrangebyscore(storage_key, min=0, max=max_score, start=start, num=results_per_page)
    redis_results.reverse()
    return [factory_object(ct_obj_string) for ct_obj_string in redis_results]



import psycopg2.extras
import uuid

from django.db import models

from ohashi.db.models.fields import CharField
from south.modelsinspector import add_introspection_rules


# Register this adapter so we can use UUID objects.
psycopg2.extras.register_uuid()


class UUIDField(CharField):
    """
    A field which stores a UUID value in hex format. This may also have the
    Boolean attribute 'auto' which will set the value on initial save to a new
    UUID value (calculated using the UUID4 method).

    """
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('default', uuid.uuid4)
        kwargs.setdefault('editable', not kwargs.get('primary_key', False))
        super(UUIDField, self).__init__(*args, **kwargs)

    def db_type(self, connection):
        return 'uuid'

    def get_db_prep_value(self, value, connection, prepared=False):
        return self.to_python(value)

    def to_python(self, value):
        if not value:
            return None
        if not isinstance(value, uuid.UUID):
            value = uuid.UUID(value)
        return value


# South custom field introspection rules.
add_introspection_rules([([UUIDField], [], {'auto': ['auto', {'default': 'False'}]},)], [r'^ohashi\.db\.fields\.UUIDField'])



#/usr/bin/env python
import os
import sys

from setuptools import setup, find_packages


if 'publish' in sys.argv:
    os.system('python setup.py sdist upload')
    sys.exit()

# Load package meta from the pkgmeta module.
pkgmeta = {}
execfile(os.path.join(os.path.dirname(__file__), 'ohashi', 'pkgmeta.py'), pkgmeta)

setup(
    name='ohashi',
    version=pkgmeta['__version__'],
    description='Yet another opinionated utilities kit for Django projects.',
    author=pkgmeta['__author__'],
    author_email='bryan@revyver.com',
    license='BSD',
    url='http://github.com/revyver/ohashi',
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'django>=1.4.1',
        'redis>=2.6.0',
        'South>=0.7.6'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Utilities'
    ]
)



from django.shortcuts import _get_queryset


def get_object_or_none(klass, *args, **kwargs):
    """
    Uses get() to return an object or None if the object does not exist.

    klass may be a Model, Manager, or QuerySet object. All other passed
    arguments and keyword arguments are used in the get() query.

    Note: Like with get(), a MultipleObjectsReturned will be raised if more
    than one object is found.

    """
    # Source: https://bitbucket.org/offline/django-annoying
    queryset = _get_queryset(klass)
    try:
        return queryset.get(*args, **kwargs)
    except queryset.model.DoesNotExist:
        return None


def get_latest_or_none(klass, *args, **kwargs):
    """
    Uses get() to return an object, or returns None if the object does not
    exist.

    klass may be a Model, Manager, or QuerySet object. All other passed
    arguments and keyword arguments are used in the get() query.

    Note: Like with get(), an MultipleObjectsReturned will be raised if more
    than one object is found.

    """
    queryset = _get_queryset(klass)
    try:
        return queryset.filter(*args, **kwargs).latest()
    except queryset.model.DoesNotExist:
        return None
